;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; cpp.el --- Generate C++ source and header files from template
;;            Also generate method prototypes.
;;
;; Author    : Mongoose <stu7440@westga.edu>
;; Maintainer: Mongoose <stu7440@westga.edu>
;;
;;--History-----------------------------------------------------
;;
;; 2001.06.01:
;; Mongoose - Created, based on my old sed script
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;; Configure your template data here
(setq template-user "Mongoose")
(setq template-author "Terry \'Mongoose\' Hendrix II")
(setq template-email "stu7440@westga.edu")
(setq template-url "http:\/\/www.westga.edu\/~stu7440\/")
(setq template-license (concat "No use w\/o permission (C) " (format-time-string "%Y") " " template-user))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(setq template-filename-cpp-h "~/scripts/templates/template.h")
(setq template-filename-cpp-cpp "~/scripts/templates/template.cpp")
(setq template-filename-cpp-method "~/scripts/templates/template-method.cpp")
(setq template-filename-cpp-line-comment "~/scripts/templates/template-line-comment.cpp")

(setq template-filename-java-java "~/scripts/templates/template.java")

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(defun template-replace ()
  ;; Get commonly used autogenerated strings
  (setq fill-in-year (format-time-string "%Y"))
  (setq fill-in-month (format-time-string "%m"))
  (setq fill-in-day (format-time-string "%d"))

  ;; Fill in the source template, note we replace from start 
  ;;  of buffer each time!
  (perform-replace "#project" project nil nil nil)
  (goto-char (point-min))
  (perform-replace "#class" classname nil nil nil)
  (goto-char (point-min))
  (perform-replace "#comment" description nil nil nil)
  (goto-char (point-min))
  (perform-replace "#author" template-author nil nil nil)
  (goto-char (point-min))
  (perform-replace "#license" template-license nil nil nil)
  (goto-char (point-min))
  (perform-replace "#email" template-email nil nil nil)
  (goto-char (point-min))
  (perform-replace "#url" template-url nil nil nil)
  (goto-char (point-min))
  (perform-replace "#user" template-user nil nil nil)
  (goto-char (point-min))
  (perform-replace "#year" fill-in-year nil nil nil)
  (goto-char (point-min))
  (perform-replace "#month" fill-in-month nil nil nil)
  (goto-char (point-min))
  (perform-replace "#day" fill-in-day nil nil nil)
  (goto-char (point-min)))


(defun cpp-class-template (project classname description)
  ;; Identify script
  "Mongoose's C++ template generator v0.0.3."

  ;; Interactive prompt for project, classname, and description
  (interactive "sProject: \nsClassname: \nsDescription: ")
  (message "Composing header and source for class %s." classname)

  ;;;;; Source file ;;;;;;;;;;;;;;;

  ;; Create source filename
  (setq filename-cpp (concat classname ".cpp"))

  ;; Generate and switch to a new buffer
  (setq buffer-cpp (generate-new-buffer filename-cpp))
  (switch-to-buffer buffer-cpp)

  ;; Insert template file contents into source buffer
  (insert-file-contents template-filename-cpp-cpp) ; nil 0 50000)

  ;; Discover new modes
  (set-auto-mode)

  ;; Fill in template with user defaults
  (template-replace)

  ;; Write out the source file if that name isn't already in use
  (if (file-readable-p filename-cpp)
      (message (concat "File " filename-cpp "exists"))
    (write-file filename-cpp))

  ;; Syntax highlighting
  (font-lock-mode)

  ;;;;; Header file ;;;;;;;;;;;;;;;

  ;; Create header filename
  (setq filename-h (concat classname ".h"))

  ;; Generate and switch to a new buffer
  (setq buffer-h (generate-new-buffer filename-h))
  (switch-to-buffer buffer-h)

  ;; Insert template file contents into header buffer
  (insert-file-contents template-filename-cpp-h) ; nil 0 50000)

  ;; Discover new modes
  (set-auto-mode)

  ;; Fill in template with user defaults
  (template-replace)

  ;; Write out the header file if that name isn't already in use
  (if (file-readable-p filename-h)
      (message (concat "File " filename-h "exists"))
    (write-file filename-h))

  ;; Syntax highlighting
  (font-lock-mode)

  ;; Output result in message
  (message "Made %s and %s" filename-cpp filename-h))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(defun cpp-method-template (method args pre post)
  ;; Identify script
  "Mongoose's C++ template generator v0.0.3."

  ;; Interactive prompt for method, args, pre, and post
  (interactive "sMethod: \nsArgs: \nsPre: \nsPost: ")

  ;; Insert template method contents into buffer
  (insert-file-contents template-filename-cpp-method)

  ;; Fill in template with user defaults

  ;; Get commonly used autogenerated strings
  (setq fill-in-year (format-time-string "%Y"))
  (setq fill-in-month (format-time-string "%m"))
  (setq fill-in-day (format-time-string "%d"))

  ;; Fill in method, do method name last to point there
  (perform-replace "#args" args nil nil nil)
  (goto-char (point-min))
  (perform-replace "#pre" pre nil nil nil)
  (goto-char (point-min))
  (perform-replace "#post" post nil nil nil)
  (goto-char (point-min))
  (perform-replace "#user" template-user nil nil nil)
  (goto-char (point-min))
  (perform-replace "#year" fill-in-year nil nil nil)
  (goto-char (point-min))
  (perform-replace "#month" fill-in-month nil nil nil)
  (goto-char (point-min))
  (perform-replace "#day" fill-in-day nil nil nil)
  (goto-char (point-min))
  (perform-replace "#method" method nil nil nil)

  (message "Sorry method skeleton not added to source file yet"))

;(global-set-key "\C-cpp" 'cpp-class-template)
;(cpp-class-template "Project Test" "TestClass" "This is the test class")

;(provide 'cpp)


(defun cpp-line-comment (comment)
  ;; Identify script
  "Mongoose's C++ template generator v0.0.3."

  ;; Interactive prompt for comment
  (interactive "sComment: ")

  ;; Get commonly used autogenerated strings
  (setq fill-in-year (format-time-string "%Y"))
  (setq fill-in-month (format-time-string "%m"))
  (setq fill-in-day (format-time-string "%d"))

  ;; Insert template method contents into buffer
  (insert-file-contents template-filename-cpp-line-comment)

  ;; Fill in the method template, note we replace from start 
  ;;  of buffer each time!
  (perform-replace "#user" template-user nil nil nil)
  (goto-char (point-min))
  (perform-replace "#year" fill-in-year nil nil nil)
  (goto-char (point-min))
  (perform-replace "#month" fill-in-month nil nil nil)
  (goto-char (point-min))
  (perform-replace "#day" fill-in-day nil nil nil)
  (goto-char (point-min))
  (perform-replace "#comment" comment nil nil nil)

  (message "Date stamped line comment generated"))



;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;


(defun cpp-session (session)
  ;; Identify script
  "Mongoose's C++ session loader v0.0.0"

  ;; Interactive prompt for session name
  (interactive "sSession: ")
  (message "Loading files in %s." session)

  (setq filename "/tmp/test")
  (setq buffer (generate-new-buffer filename))
  (switch-to-buffer buffer)
  (insert-file-contents filename))


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;




(defun java-class-template (project classname description)
  ;; Identify script
  "Mongoose's C++ template generator v0.0.3."

  ;; Interactive prompt for project, classname, and description
  (interactive "sProject: \nsClassname: \nsDescription: ")
  (message "Composing header and source for class %s." classname)

  ;;;;; Source file ;;;;;;;;;;;;;;;

  ;; Create source filename
  (setq filename-java (concat classname ".java"))

  ;; Generate and switch to a new buffer
  (setq buffer-java (generate-new-buffer filename-java))
  (switch-to-buffer buffer-java)

  ;; Insert template file contents into source buffer
  (insert-file-contents template-filename-java-java)

  ;; Discover new modes
  (set-auto-mode)

  ;; Fill in template
  (template-replace)

  ;; Write out the source file if that name isn't already in use
  (if (file-readable-p filename-java)
      (message (concat "File " filename-java "exists"))
    (write-file filename-java))

  ;; Syntax Highlighting
  (font-lock-mode)

  ;; Output result in message
  (message "Made %s" filename-java))
